# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
import json
import re
import os

from eve.utils import ParsedRequest
from superdesk.utc import utcnow
from datetime import timedelta

from apps.content_filters.filter_condition.filter_condition_service import FilterConditionService
from apps.content_filters.filter_condition.filter_condition import FilterCondition
from apps.content_filters.filter_condition.filter_condition_operator import FilterConditionOperator
from superdesk import get_resource_service
from superdesk.tests import TestCase
from superdesk.vocabularies.commands import VocabulariesPopulateCommand


class FilterConditionTests(TestCase):
    def setUp(self):
        self.req = ParsedRequest()
        with self.app.test_request_context(self.app.config.get('URL_PREFIX')):
            self.articles = [{'_id': '1', 'urgency': 1, 'headline': 'story', 'state': 'fetched'},
                             {'_id': '2', 'headline': 'prtorque', 'state': 'fetched'},
                             {'_id': '3', 'urgency': 3, 'state': 'fetched', 'flags': {'marked_for_sms': True}},
                             {'_id': '4', 'urgency': 4, 'state': 'fetched', 'task': {'desk': '1'},
                              'ingest_provider': '1'},
                             {'_id': '5', 'urgency': 2, 'state': 'fetched', 'task': {'desk': '2'}, 'priority': 3},
                             {'_id': '6', 'state': 'fetched', 'embargo': utcnow(),
                             'schedule_settings': {'utc_embargo': utcnow() + timedelta(minutes=20)}},
                             {'_id': '7', 'genre': [{'name': 'Sidebar'}], 'state': 'fetched'},
                             {'_id': '8', 'subject': [{'name': 'adult education',
                                                       'qcode': '05001000',
                                                       'parent': '05000000'},
                                                      {'name': 'high schools',
                                                       'qcode': '05005003',
                                                       'parent': '05005000'}], 'state': 'fetched'},
                             {'_id': '9', 'state': 'fetched', 'anpa_category':
                                 [{'qcode': 'a', 'name': 'Aus News'}]},
                             {'_id': '10', 'body_html': '<p>Mention<p>', 'embargo': utcnow(),
                             'schedule_settings': {'utc_embargo': utcnow() - timedelta(minutes=20)}},
                             {'_id': '11', 'place': [{'qcode': 'NSW', 'name': 'NSW'}], 'state': 'fetched'}]

            self.app.data.insert('archive', self.articles)

            self.app.data.insert('filter_conditions',
                                 [{'_id': 1,
                                   'field': 'headline',
                                   'operator': 'like',
                                   'value': 'tor',
                                   'name': 'test-1'}])
            self.app.data.insert('filter_conditions',
                                 [{'_id': 2,
                                   'field': 'urgency',
                                   'operator': 'in',
                                   'value': '2',
                                   'name': 'test-2'}])
            self.app.data.insert('filter_conditions',
                                 [{'_id': 3,
                                   'field': 'urgency',
                                   'operator': 'in',
                                   'value': '3,4,5',
                                   'name': 'test-2'}])
            self.app.data.insert('filter_conditions',
                                 [{'_id': 4,
                                   'field': 'urgency',
                                   'operator': 'nin',
                                   'value': '1,2,3',
                                   'name': 'test-2'}])
            self.app.data.insert('filter_conditions',
                                 [{'_id': 5,
                                   'field': 'urgency',
                                   'operator': 'in',
                                   'value': '2,5',
                                   'name': 'test-2'}])
            self.app.data.insert('content_filters',
                                 [{"_id": 1,
                                   "content_filter": [{"expression": {"fc": [1]}}],
                                   "name": "soccer-only"}])

    def _setup_elastic_args(self, elastic_translation, search_type='filter'):
        if search_type == 'keyword':
            self.req.args = {'source': json.dumps({'query': {'bool': {'should': [elastic_translation]}}})}
        elif search_type == 'not':
            self.req.args = {'source': json.dumps({'query': {'bool': {'must_not': [elastic_translation]}}})}
        elif search_type == 'filter':
            self.req.args = {'source': json.dumps({'query': {
                'filtered': {
                    'filter': {
                        'bool': {
                            'should': [elastic_translation]}}}}})}
        elif search_type == 'match':
            self.req.args = {'source': json.dumps({'query': {
                'filtered': {
                    'query': {
                        'bool': {
                            'should': [{
                                'bool': {
                                    'must': [elastic_translation]}}]}}}}})}

    def test_mongo_using_genre_filter_complete_string(self):
        f = FilterCondition('genre', 'in', 'Sidebar')
        query = f.get_mongo_query()
        with self.app.app_context():
            docs = get_resource_service('archive'). \
                get_from_mongo(req=self.req, lookup=query)
            self.assertEqual(1, docs.count())
            self.assertEqual('7', docs[0]['_id'])

    def test_mongo_using_priority_compare_eq_filter(self):
        f = FilterCondition('priority', 'eq', '3')
        query = f.get_mongo_query()
        with self.app.app_context():
            docs = get_resource_service('archive'). \
                get_from_mongo(req=self.req, lookup=query)
            self.assertEqual(1, docs.count())
            self.assertEqual('5', docs[0]['_id'])

    def test_mongo_using_priority_compare_ne_filter(self):
        f = FilterCondition('priority', 'ne', '3')
        query = f.get_mongo_query()
        with self.app.app_context():
            docs = get_resource_service('archive'). \
                get_from_mongo(req=self.req, lookup=query)
            self.assertEqual(10, docs.count())
            self.assertTrue('5' not in [d['_id'] for d in docs])

    def test_mongo_using_priority_compare_lte_filter(self):
        f = FilterCondition('priority', 'lte', '3')
        query = f.get_mongo_query()
        with self.app.app_context():
            docs = get_resource_service('archive'). \
                get_from_mongo(req=self.req, lookup=query)
            self.assertEqual(1, docs.count())
            self.assertEqual('5', docs[0]['_id'])

    def test_mongo_using_desk_filter_complete_string(self):
        f = FilterCondition('desk', 'in', '1')
        query = f.get_mongo_query()
        with self.app.app_context():
            docs = get_resource_service('archive'). \
                get_from_mongo(req=self.req, lookup=query)
            self.assertEqual(1, docs.count())
            self.assertEqual('4', docs[0]['_id'])

    def test_mongo_using_desk_filter_nin(self):
        f = FilterCondition('desk', 'nin', '1')
        query = f.get_mongo_query()
        with self.app.app_context():
            docs = get_resource_service('archive'). \
                get_from_mongo(req=self.req, lookup=query)
            self.assertEqual(10, docs.count())

    def test_mongo_using_desk_filter_eq(self):
        f = FilterCondition('desk', 'eq', '1')
        query = f.get_mongo_query()
        with self.app.app_context():
            docs = get_resource_service('archive'). \
                get_from_mongo(req=self.req, lookup=query)
            self.assertEqual(1, docs.count())
            self.assertEqual('4', docs[0]['_id'])

    def test_mongo_using_sms_filter_with_is(self):
        f = FilterCondition('sms', 'in', 'true')
        query = f.get_mongo_query()
        with self.app.app_context():
            docs = get_resource_service('archive'). \
                get_from_mongo(req=self.req, lookup=query)
            self.assertEqual(1, docs.count())

    def test_mongo_using_sms_filter_with_eq_comp(self):
        f = FilterCondition('sms', 'eq', 'true')
        query = f.get_mongo_query()
        with self.app.app_context():
            docs = get_resource_service('archive'). \
                get_from_mongo(req=self.req, lookup=query)
            self.assertEqual(1, docs.count())

    def test_mongo_using_embargo_filter_with_eq_comp(self):
        f = FilterCondition('embargo', 'eq', 'true')
        query = f.get_mongo_query()
        with self.app.app_context():
            docs = get_resource_service('archive'). \
                get_from_mongo(req=self.req, lookup=query)
            self.assertEqual(1, docs.count())

    def test_mongo_using_desk_filter_in_list(self):
        f = FilterCondition('desk', 'in', '1,2')
        query = f.get_mongo_query()
        with self.app.app_context():
            docs = get_resource_service('archive'). \
                get_from_mongo(req=self.req, lookup=query)
            self.assertEqual(2, docs.count())

    def test_mongo_using_category_filter_complete_string(self):
        f = FilterCondition('anpa_category', 'in', 'a,i')
        query = f.get_mongo_query()
        with self.app.app_context():
            docs = get_resource_service('archive'). \
                get_from_mongo(req=self.req, lookup=query)
            self.assertEqual(1, docs.count())
            self.assertEqual('9', docs[0]['_id'])

    def test_mongo_using_subject_filter_complete_string(self):
        f = FilterCondition('subject', 'in', '05005003')
        query = f.get_mongo_query()
        with self.app.app_context():
            docs = get_resource_service('archive'). \
                get_from_mongo(req=self.req, lookup=query)
            self.assertEqual(1, docs.count())
            self.assertEqual('8', docs[0]['_id'])

    def test_mongo_using_subject_filter_complete_string_eq(self):
        f = FilterCondition('subject', 'eq', '05005003')
        query = f.get_mongo_query()
        with self.app.app_context():
            docs = get_resource_service('archive'). \
                get_from_mongo(req=self.req, lookup=query)
            self.assertEqual(1, docs.count())
            self.assertEqual('8', docs[0]['_id'])

    def test_mongo_using_like_filter_complete_string(self):
        f = FilterCondition('headline', 'like', 'story')
        query = f.get_mongo_query()
        with self.app.app_context():
            docs = get_resource_service('archive'). \
                get_from_mongo(req=self.req, lookup=query)
            self.assertEqual(1, docs.count())
            self.assertEqual('1', docs[0]['_id'])

    def test_mongo_using_like_filter_complete_string_eq(self):
        f = FilterCondition('headline', 'like', 'story')
        query = f.get_mongo_query()
        with self.app.app_context():
            docs = get_resource_service('archive'). \
                get_from_mongo(req=self.req, lookup=query)
            self.assertEqual(1, docs.count())
            self.assertEqual('1', docs[0]['_id'])

    def test_mongo_using_like_filter_partial_string(self):
        f = FilterCondition('headline', 'like', 'tor')
        query = f.get_mongo_query()
        with self.app.app_context():
            docs = get_resource_service('archive'). \
                get_from_mongo(req=self.req, lookup=query)
            doc_ids = [d['_id'] for d in docs]
            self.assertEqual(2, docs.count())
            self.assertTrue('1' in doc_ids)
            self.assertTrue('2' in doc_ids)

    def test_mongo_using_startswith_filter(self):
        f = FilterCondition('headline', 'startswith', 'Sto')
        query = f.get_mongo_query()
        with self.app.app_context():
            docs = get_resource_service('archive'). \
                get_from_mongo(req=self.req, lookup=query)
            self.assertEqual(1, docs.count())
            self.assertEqual('1', docs[0]['_id'])

    def test_mongo_using_endswith_filter(self):
        f = FilterCondition('headline', 'endswith', 'Que')
        query = f.get_mongo_query()
        with self.app.app_context():
            docs = get_resource_service('archive'). \
                get_from_mongo(req=self.req, lookup=query)
            self.assertEqual(1, docs.count())
            self.assertEqual('2', docs[0]['_id'])

    def test_mongo_using_notlike_filter(self):
        f = FilterCondition('headline', 'notlike', 'Que')
        query = f.get_mongo_query()
        with self.app.app_context():
            docs = get_resource_service('archive'). \
                get_from_mongo(req=self.req, lookup=query)
            self.assertEqual(10, docs.count())
            doc_ids = [d['_id'] for d in docs]
            self.assertTrue('2' not in doc_ids)

    def test_mongo_using_in_filter(self):
        f = FilterCondition('urgency', 'in', '3,4')
        query = f.get_mongo_query()
        with self.app.app_context():
            docs = get_resource_service('archive'). \
                get_from_mongo(req=self.req, lookup=query)
            self.assertEqual(2, docs.count())
            self.assertEqual('3', docs[0]['_id'])
            self.assertEqual('4', docs[1]['_id'])

    def test_mongo_using_notin_filter(self):
        f = FilterCondition('urgency', 'nin', '2,3,4')
        query = f.get_mongo_query()
        with self.app.app_context():
            docs = get_resource_service('archive'). \
                get_from_mongo(req=self.req, lookup=query)
            self.assertEqual(8, docs.count())
            doc_ids = [d['_id'] for d in docs]
            self.assertTrue('1' in doc_ids)
            self.assertTrue('2' in doc_ids)

    def test_elastic_using_genre_filter_complete_string(self):
        f = FilterCondition('genre', 'in', 'Sidebar')
        query = f.get_elastic_query()
        with self.app.app_context():
            self._setup_elastic_args(query)
            docs = get_resource_service('archive').get(req=self.req, lookup=None)
            doc_ids = [d['_id'] for d in docs]
            self.assertEqual(1, docs.count())
            self.assertTrue('7' in doc_ids)

    def test_elastic_using_sms_filter(self):
        f = FilterCondition('sms', 'in', 'true')
        query = f.get_elastic_query()
        with self.app.app_context():
            self._setup_elastic_args(query)
            docs = get_resource_service('archive').get(req=self.req, lookup=None)
            doc_ids = [d['_id'] for d in docs]
            self.assertEqual(1, docs.count())
            self.assertTrue('3' in doc_ids)

    def test_elastic_using_embargo_filter(self):
        f = FilterCondition('embargo', 'eq', 'true')
        query = f.get_elastic_query()
        with self.app.app_context():
            self._setup_elastic_args(query)
            docs = get_resource_service('archive').get(req=self.req, lookup=None)
            doc_ids = [d['_id'] for d in docs]
            self.assertEqual(1, docs.count())
            self.assertTrue('6' in doc_ids)

    def test_elastic_using_subject_filter_complete_string(self):
        f = FilterCondition('subject', 'in', '05005003')
        query = f.get_elastic_query()
        with self.app.app_context():
            self._setup_elastic_args(query)
            docs = get_resource_service('archive').get(req=self.req, lookup=None)
            doc_ids = [d['_id'] for d in docs]
            self.assertEqual(1, docs.count())
            self.assertTrue('8' in doc_ids)

    def test_elastic_using_anpa_category_filter_complete_string(self):
        f = FilterCondition('anpa_category', 'in', 'a,i')
        query = f.get_elastic_query()
        with self.app.app_context():
            self._setup_elastic_args(query)
            docs = get_resource_service('archive').get(req=self.req, lookup=None)
            doc_ids = [d['_id'] for d in docs]
            self.assertEqual(1, docs.count())
            self.assertTrue('9' in doc_ids)

    def test_elastic_using_in_filter(self):
        f = FilterCondition('urgency', 'in', '3,4')
        query = f.get_elastic_query()
        with self.app.app_context():
            self._setup_elastic_args(query)
            docs = get_resource_service('archive').get(req=self.req, lookup=None)
            doc_ids = [d['_id'] for d in docs]
            self.assertEqual(2, docs.count())
            self.assertTrue('4' in doc_ids)
            self.assertTrue('3' in doc_ids)

    def test_elastic_using_eq_filter(self):
        f = FilterCondition('urgency', 'eq', '3')
        query = f.get_elastic_query()
        with self.app.app_context():
            self._setup_elastic_args(query)
            docs = get_resource_service('archive').get(req=self.req, lookup=None)
            doc_ids = [d['_id'] for d in docs]
            self.assertEqual(1, docs.count())
            self.assertTrue('3' in doc_ids)

    def test_elastic_using_nin_filter(self):
        f = FilterCondition('urgency', 'nin', '3,4')
        query = f.get_elastic_query()
        with self.app.app_context():
            self._setup_elastic_args(query, 'not')
            docs = get_resource_service('archive').get(req=self.req, lookup=None)
            self.assertEqual(8, docs.count())
            doc_ids = [d['_id'] for d in docs]
            self.assertTrue('6' in doc_ids)
            self.assertTrue('5' in doc_ids)

    def test_elastic_using_ne_filter(self):
        f = FilterCondition('urgency', 'ne', '4')
        query = f.get_elastic_query()
        with self.app.app_context():
            self._setup_elastic_args(query, 'not')
            docs = get_resource_service('archive').get(req=self.req, lookup=None)
            self.assertEqual(9, docs.count())
            doc_ids = [d['_id'] for d in docs]
            self.assertTrue('6' in doc_ids)
            self.assertTrue('5' in doc_ids)

    def test_elastic_using_like_filter(self):
        f = FilterCondition('headline', 'like', 'Tor')
        query = f.get_elastic_query()
        with self.app.app_context():
            self._setup_elastic_args(query, 'keyword')
            docs = get_resource_service('archive').get(req=self.req, lookup=None)
            self.assertEqual(2, docs.count())
            doc_ids = [d['_id'] for d in docs]
            self.assertTrue('1' in doc_ids)
            self.assertTrue('2' in doc_ids)

    def test_elastic_using_notlike_filter(self):
        f = FilterCondition('headline', 'notlike', 'que')
        query = f.get_elastic_query()
        with self.app.app_context():
            self._setup_elastic_args(query, 'not')
            docs = get_resource_service('archive').get(req=self.req, lookup=None)
            self.assertEqual(9, docs.count())
            doc_ids = [d['_id'] for d in docs]
            self.assertTrue('2' not in doc_ids)

    def test_elastic_using_startswith_filter(self):
        f = FilterCondition('headline', 'startswith', 'Sto')
        query = f.get_elastic_query()
        with self.app.app_context():
            self._setup_elastic_args(query, 'keyword')
            docs = get_resource_service('archive').get(req=self.req, lookup=None)
            self.assertEqual(1, docs.count())
            self.assertEqual('1', docs[0]['_id'])

    def test_elastic_using_endswith_filter(self):
        f = FilterCondition('headline', 'endswith', 'Que')
        query = f.get_elastic_query()
        with self.app.app_context():
            self._setup_elastic_args(query, 'keyword')
            docs = get_resource_service('archive').get(req=self.req, lookup=None)
            self.assertEqual(1, docs.count())
            self.assertEqual('2', docs[0]['_id'])

    def test_get_mongo_operator(self):
        self.assertEqual(FilterConditionOperator.factory('in').mongo_operator, '$in')
        self.assertEqual(FilterConditionOperator.factory('nin').mongo_operator, '$nin')
        self.assertEqual(FilterConditionOperator.factory('like').mongo_operator, '$regex')
        self.assertEqual(FilterConditionOperator.factory('notlike').mongo_operator, '$not')
        self.assertEqual(FilterConditionOperator.factory('startswith').mongo_operator, '$regex')
        self.assertEqual(FilterConditionOperator.factory('endswith').mongo_operator, '$regex')

    def test_get_mongo_value(self):
        f = FilterCondition('urgency', 'in', '1,2')
        self.assertEqual(f.value.get_mongo_value(f.field), [1, 2])

        f = FilterCondition('priority', 'nin', '3')
        self.assertEqual(f.value.get_mongo_value(f.field), [3])

        f = FilterCondition('headline', 'like', 'test')
        self.assertEqual(f.value.get_mongo_value(f.field), re.compile('.*test.*', re.IGNORECASE))

        f = FilterCondition('headline', 'notlike', 'test')
        self.assertEqual(f.value.get_mongo_value(f.field), re.compile('.*test.*', re.IGNORECASE))

        f = FilterCondition('headline', 'startswith', 'test')
        self.assertEqual(f.value.get_mongo_value(f.field), re.compile('^test', re.IGNORECASE))

        f = FilterCondition('headline', 'endswith', 'test')
        self.assertEqual(f.value.get_mongo_value(f.field), re.compile('.*test', re.IGNORECASE))

    def test_does_match_with_eq(self):
        f = FilterCondition('urgency', 'eq', '1')
        self.assertTrue(f.does_match(self.articles[0]))
        self.assertFalse(f.does_match(self.articles[1]))
        self.assertFalse(f.does_match(self.articles[2]))
        self.assertFalse(f.does_match(self.articles[3]))
        self.assertFalse(f.does_match(self.articles[4]))
        self.assertFalse(f.does_match(self.articles[5]))

    def test_does_match_with_eq_string(self):
        f = FilterCondition('headline', 'eq', 'Story')
        self.assertTrue(f.does_match(self.articles[0]))
        self.assertFalse(f.does_match(self.articles[1]))
        self.assertFalse(f.does_match(self.articles[2]))
        self.assertFalse(f.does_match(self.articles[3]))
        self.assertFalse(f.does_match(self.articles[4]))
        self.assertFalse(f.does_match(self.articles[5]))

    def test_does_match_with_eq_bool(self):
        f = FilterCondition('sms', 'eq', 'true')
        self.assertFalse(f.does_match(self.articles[0]))
        self.assertFalse(f.does_match(self.articles[1]))
        self.assertTrue(f.does_match(self.articles[2]))
        self.assertFalse(f.does_match(self.articles[3]))
        self.assertFalse(f.does_match(self.articles[4]))
        self.assertFalse(f.does_match(self.articles[5]))
        self.assertFalse(f.does_match(self.articles[6]))
        self.assertFalse(f.does_match(self.articles[7]))

    def test_does_match_with_lt_int(self):
        f = FilterCondition('priority', 'lt', '7')
        self.assertFalse(f.does_match(self.articles[0]))
        self.assertFalse(f.does_match(self.articles[1]))
        self.assertFalse(f.does_match(self.articles[2]))
        self.assertFalse(f.does_match(self.articles[3]))
        self.assertTrue(f.does_match(self.articles[4]))
        self.assertFalse(f.does_match(self.articles[5]))
        self.assertFalse(f.does_match(self.articles[6]))
        self.assertFalse(f.does_match(self.articles[7]))

    def test_does_match_with_gt_int(self):
        f = FilterCondition('priority', 'gt', '1')
        self.assertFalse(f.does_match(self.articles[0]))
        self.assertFalse(f.does_match(self.articles[1]))
        self.assertFalse(f.does_match(self.articles[2]))
        self.assertFalse(f.does_match(self.articles[3]))
        self.assertTrue(f.does_match(self.articles[4]))
        self.assertFalse(f.does_match(self.articles[5]))
        self.assertFalse(f.does_match(self.articles[6]))
        self.assertFalse(f.does_match(self.articles[7]))

    def test_does_match_with_gte_int(self):
        f = FilterCondition('priority', 'gte', '3')
        self.assertFalse(f.does_match(self.articles[0]))
        self.assertFalse(f.does_match(self.articles[1]))
        self.assertFalse(f.does_match(self.articles[2]))
        self.assertFalse(f.does_match(self.articles[3]))
        self.assertTrue(f.does_match(self.articles[4]))
        self.assertFalse(f.does_match(self.articles[5]))
        self.assertFalse(f.does_match(self.articles[6]))
        self.assertFalse(f.does_match(self.articles[7]))

    def test_does_match_with_ne_int(self):
        f = FilterCondition('priority', 'ne', '3')
        self.assertTrue(f.does_match(self.articles[0]))
        self.assertTrue(f.does_match(self.articles[1]))
        self.assertTrue(f.does_match(self.articles[2]))
        self.assertTrue(f.does_match(self.articles[3]))
        self.assertFalse(f.does_match(self.articles[4]))
        self.assertTrue(f.does_match(self.articles[5]))
        self.assertTrue(f.does_match(self.articles[6]))
        self.assertTrue(f.does_match(self.articles[7]))

    def test_does_match_with_like_full(self):
        f = FilterCondition('headline', 'like', 'story')
        self.assertTrue(f.does_match(self.articles[0]))
        self.assertFalse(f.does_match(self.articles[1]))
        self.assertFalse(f.does_match(self.articles[2]))
        self.assertFalse(f.does_match(self.articles[3]))
        self.assertFalse(f.does_match(self.articles[4]))
        self.assertFalse(f.does_match(self.articles[5]))

    def test_does_match_with_like_partial(self):
        f = FilterCondition('headline', 'like', 'tor')
        self.assertTrue(f.does_match(self.articles[0]))
        self.assertTrue(f.does_match(self.articles[1]))
        self.assertFalse(f.does_match(self.articles[2]))
        self.assertFalse(f.does_match(self.articles[3]))
        self.assertFalse(f.does_match(self.articles[4]))
        self.assertFalse(f.does_match(self.articles[5]))

    def test_does_match_with_startswith_filter(self):
        f = FilterCondition('headline', 'startswith', 'Sto')
        self.assertTrue(f.does_match(self.articles[0]))
        self.assertFalse(f.does_match(self.articles[1]))
        self.assertFalse(f.does_match(self.articles[2]))
        self.assertFalse(f.does_match(self.articles[3]))
        self.assertFalse(f.does_match(self.articles[4]))
        self.assertFalse(f.does_match(self.articles[5]))

    def test_does_match_with_startswith_filter_html_field(self):
        f = FilterCondition('body_html', 'startswith', 'men')
        self.assertTrue(f.does_match(self.articles[9]))

    def test_does_match_with_endswith_filter(self):
        f = FilterCondition('headline', 'endswith', 'Que')
        self.assertFalse(f.does_match(self.articles[0]))
        self.assertTrue(f.does_match(self.articles[1]))
        self.assertFalse(f.does_match(self.articles[2]))
        self.assertFalse(f.does_match(self.articles[3]))
        self.assertFalse(f.does_match(self.articles[4]))
        self.assertFalse(f.does_match(self.articles[5]))

    def test_does_match_with_notlike_filter(self):
        f = FilterCondition('headline', 'notlike', 'Que')
        self.assertTrue(f.does_match(self.articles[0]))
        self.assertFalse(f.does_match(self.articles[1]))
        self.assertTrue(f.does_match(self.articles[2]))
        self.assertTrue(f.does_match(self.articles[3]))
        self.assertTrue(f.does_match(self.articles[4]))
        self.assertTrue(f.does_match(self.articles[5]))

    def test_does_match_with_genre_filter(self):
        f = FilterCondition('genre', 'in', 'Sidebar')
        self.assertFalse(f.does_match(self.articles[0]))
        self.assertFalse(f.does_match(self.articles[1]))
        self.assertFalse(f.does_match(self.articles[2]))
        self.assertFalse(f.does_match(self.articles[3]))
        self.assertFalse(f.does_match(self.articles[4]))
        self.assertFalse(f.does_match(self.articles[5]))
        self.assertTrue(f.does_match(self.articles[6]))
        self.assertFalse(f.does_match(self.articles[7]))
        self.assertFalse(f.does_match({'genre': None}))
        self.assertTrue(f.does_match({'genre': [{'name': 'Sidebar'}]}))
        self.assertFalse(f.does_match({'genre': [{'name': 'Article'}]}))
        self.assertTrue(f.does_match({'genre': [{'name': 'Sidebar'}, {'name': 'Article'}]}))

    def test_does_match_with_category_filter(self):
        f = FilterCondition('anpa_category', 'in', 'a,i')
        self.assertFalse(f.does_match(self.articles[0]))
        self.assertFalse(f.does_match(self.articles[1]))
        self.assertFalse(f.does_match(self.articles[2]))
        self.assertFalse(f.does_match(self.articles[3]))
        self.assertFalse(f.does_match(self.articles[4]))
        self.assertFalse(f.does_match(self.articles[5]))
        self.assertFalse(f.does_match(self.articles[6]))
        self.assertFalse(f.does_match(self.articles[7]))
        self.assertTrue(f.does_match(self.articles[8]))

    def test_does_match_with_subject_filter(self):
        f = FilterCondition('subject', 'in', '05005003')
        self.assertFalse(f.does_match(self.articles[0]))
        self.assertFalse(f.does_match(self.articles[1]))
        self.assertFalse(f.does_match(self.articles[2]))
        self.assertFalse(f.does_match(self.articles[3]))
        self.assertFalse(f.does_match(self.articles[4]))
        self.assertFalse(f.does_match(self.articles[5]))
        self.assertFalse(f.does_match(self.articles[6]))
        self.assertTrue(f.does_match(self.articles[7]))

    def test_does_match_with_sms_filter(self):
        f = FilterCondition('sms', 'nin', 'true')
        self.assertTrue(f.does_match(self.articles[0]))
        self.assertTrue(f.does_match(self.articles[1]))
        self.assertFalse(f.does_match(self.articles[2]))
        self.assertTrue(f.does_match(self.articles[3]))
        self.assertTrue(f.does_match(self.articles[4]))
        self.assertTrue(f.does_match(self.articles[5]))
        self.assertTrue(f.does_match(self.articles[6]))
        self.assertTrue(f.does_match(self.articles[7]))

    def test_does_match_with_embargo_filter_with_false(self):
        f = FilterCondition('embargo', 'eq', 'false')
        self.assertTrue(f.does_match(self.articles[0]))
        self.assertTrue(f.does_match(self.articles[1]))
        self.assertTrue(f.does_match(self.articles[2]))
        self.assertTrue(f.does_match(self.articles[3]))
        self.assertTrue(f.does_match(self.articles[4]))
        self.assertFalse(f.does_match(self.articles[5]))
        self.assertTrue(f.does_match(self.articles[6]))
        self.assertTrue(f.does_match(self.articles[7]))

    def test_does_match_with_embargo_filter_with_true(self):
        f = FilterCondition('embargo', 'eq', 'true')
        self.assertFalse(f.does_match(self.articles[0]))
        self.assertFalse(f.does_match(self.articles[1]))
        self.assertFalse(f.does_match(self.articles[2]))
        self.assertFalse(f.does_match(self.articles[3]))
        self.assertFalse(f.does_match(self.articles[4]))
        self.assertTrue(f.does_match(self.articles[5]))
        self.assertFalse(f.does_match(self.articles[6]))
        self.assertFalse(f.does_match(self.articles[7]))

    def test_does_match_with_in_filter(self):
        f = FilterCondition('urgency', 'in', '3,4')
        self.assertFalse(f.does_match(self.articles[0]))
        self.assertFalse(f.does_match(self.articles[1]))
        self.assertTrue(f.does_match(self.articles[2]))
        self.assertTrue(f.does_match(self.articles[3]))
        self.assertFalse(f.does_match(self.articles[4]))
        self.assertFalse(f.does_match(self.articles[5]))

    def test_does_match_with_in_filter_case_insensitive(self):
        f = FilterCondition('source', 'in', 'aap,reuters')
        self.assertTrue(f.does_match({'source': 'AAP'}))
        self.assertTrue(f.does_match({'source': 'aap'}))
        self.assertTrue(f.does_match({'source': 'REUTERS'}))
        f = FilterCondition('source', 'in', 'AAP')
        self.assertTrue(f.does_match({'source': 'AAP'}))
        self.assertTrue(f.does_match({'source': 'aap'}))
        self.assertFalse(f.does_match({'source': 'REUTERS'}))

    def test_does_match_with_nin_filter(self):
        f = FilterCondition('urgency', 'nin', '2,3,4')
        self.assertTrue(f.does_match(self.articles[0]))
        self.assertTrue(f.does_match(self.articles[1]))
        self.assertFalse(f.does_match(self.articles[2]))
        self.assertFalse(f.does_match(self.articles[3]))
        self.assertFalse(f.does_match(self.articles[4]))
        self.assertTrue(f.does_match(self.articles[5]))

    def test_are_equal1(self):
        f = FilterConditionService()
        new_doc = {'name': 'A', 'field': 'urgency', 'operator': 'nin', 'value': '2,3,4'}
        doc = {'_id': 1, 'name': 'B', 'field': 'urgency', 'operator': 'nin', 'value': '2,3,4'}
        self.assertTrue(f._are_equal(new_doc, doc))

    def test_are_equal2(self):
        f = FilterConditionService()
        new_doc = {'name': 'A', 'field': 'urgency', 'operator': 'nin', 'value': '4,2,3'}
        doc = {'_id': 1, 'name': 'B', 'field': 'urgency', 'operator': 'nin', 'value': '2,3,4'}
        self.assertTrue(f._are_equal(new_doc, doc))

    def test_are_equal3(self):
        f = FilterConditionService()
        new_doc = {'name': 'A', 'field': 'urgency', 'operator': 'nin', 'value': 'jump,track'}
        doc = {'_id': 1, 'name': 'B', 'field': 'urgency', 'operator': 'nin', 'value': 'tump,jrack'}
        self.assertTrue(f._are_equal(new_doc, doc))

    def test_are_equal4(self):
        f = FilterConditionService()
        new_doc = {'name': 'A', 'field': 'urgency', 'operator': 'nin', 'value': '4,2,3'}
        doc = {'_id': 1, 'name': 'B', 'field': 'urgency', 'operator': 'nin', 'value': '2,3'}
        self.assertFalse(f._are_equal(new_doc, doc))

    def test_if_fc_is_used(self):
        f = FilterConditionService()
        with self.app.app_context():
            self.assertTrue(f._get_referenced_filter_conditions(1).count() == 1)
            self.assertTrue(f._get_referenced_filter_conditions(2).count() == 0)

    def test_check_similar(self):
        f = get_resource_service('filter_conditions')
        filter_condition1 = {'field': 'urgency', 'operator': 'in', 'value': '2'}
        filter_condition2 = {'field': 'urgency', 'operator': 'in', 'value': '3'}
        filter_condition3 = {'field': 'urgency', 'operator': 'in', 'value': '1'}
        filter_condition4 = {'field': 'urgency', 'operator': 'in', 'value': '5'}
        filter_condition5 = {'field': 'urgency', 'operator': 'nin', 'value': '5'}
        filter_condition6 = {'field': 'headline', 'operator': 'like', 'value': 'tor'}
        with self.app.app_context():
            cmd = VocabulariesPopulateCommand()
            filename = os.path.join(os.path.abspath(
                os.path.dirname("apps/prepopulate/data_init/vocabularies.json")), "vocabularies.json")
            cmd.run(filename)
            self.assertTrue(len(f.check_similar(filter_condition1)) == 2)
            self.assertTrue(len(f.check_similar(filter_condition2)) == 1)
            self.assertTrue(len(f.check_similar(filter_condition3)) == 0)
            self.assertTrue(len(f.check_similar(filter_condition4)) == 3)
            self.assertTrue(len(f.check_similar(filter_condition5)) == 1)
            self.assertTrue(len(f.check_similar(filter_condition6)) == 1)

    def test_mongo_using_place_filter_complete_string(self):
        f = FilterCondition('place', 'in', 'NSW')
        query = f.get_mongo_query()
        with self.app.app_context():
            docs = get_resource_service('archive'). \
                get_from_mongo(req=self.req, lookup=query)
            self.assertEqual(1, docs.count())
            self.assertEqual('11', docs[0]['_id'])

    def test_elastic_using_place_filter_complete_string(self):
        f = FilterCondition('place', 'match', 'NSW')
        query = f.get_elastic_query()
        with self.app.app_context():
            self._setup_elastic_args(query, 'match')
            docs = get_resource_service('archive').get(req=self.req, lookup=None)
            doc_ids = [d['_id'] for d in docs]
            self.assertEqual(1, docs.count())
            self.assertTrue('11' in doc_ids)

    def test_mongo_using_ingest_provider_filter_eq(self):
        f = FilterCondition('ingest_provider', 'eq', '1')
        query = f.get_mongo_query()
        with self.app.app_context():
            docs = get_resource_service('archive'). \
                get_from_mongo(req=self.req, lookup=query)
            self.assertEqual(1, docs.count())
            self.assertEqual('4', docs[0]['_id'])

    def test_elastic_using_ingest_provider_filter_eq(self):
        f = FilterCondition('ingest_provider', 'eq', '1')
        query = f.get_elastic_query()
        with self.app.app_context():
            self._setup_elastic_args(query)
            docs = get_resource_service('archive').get(req=self.req, lookup=None)
            doc_ids = [d['_id'] for d in docs]
            self.assertEqual(1, docs.count())
            self.assertTrue('4' in doc_ids)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


from string import Template
from types import ModuleType
from flask import current_app
import superdesk
import getpass
import os
import re
import time
from eve.utils import ParsedRequest


DEFAULT_DATA_UPDATE_DIR_NAME = 'data_updates'
MAIN_DATA_UPDATES_DIR = os.path.abspath(os.path.join(os.path.dirname(
    os.path.realpath(__file__)), os.pardir, DEFAULT_DATA_UPDATE_DIR_NAME))
DATA_UPDATES_FILENAME_REGEX = re.compile('^(\d+).*\.py$')
DATA_UPDATE_TEMPLATE = '''
# -*- coding: utf-8; -*-
# This file is part of Superdesk.
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
#
# Author  : $user
# Creation: $current_date

from superdesk.commands.data_updates import DataUpdate


class DataUpdate(DataUpdate):

    resource = '$resource'

    def forwards(self, mongodb_collection, mongodb_database):
        $default_fw_implementation

    def backwards(self, mongodb_collection, mongodb_database):
        $default_bw_implementation
'''.lstrip('\n')
DEFAULT_DATA_UPDATE_FW_IMPLEMENTATION = 'raise NotImplementedError()'
DEFAULT_DATA_UPDATE_BW_IMPLEMENTATION = 'raise NotImplementedError()'


def get_dirs(only_relative_folder=False):
    dirs = []
    try:
        with superdesk.app.app_context():
            dirs.append(current_app.config.get('DATA_UPDATES_PATH', DEFAULT_DATA_UPDATE_DIR_NAME))
    except RuntimeError:
        # working outside of application context
        pass
    if not only_relative_folder:
        dirs.append(MAIN_DATA_UPDATES_DIR)
    assert len(dirs) <= 2
    return dirs


def get_data_updates_files(strip_file_extension=False):
    """Return the list of data updates filenames.

    .py extension can be removed with `strip_file_extension` parameter.
    """
    files = []
    # create folder if doens't exist
    for folder in get_dirs():
        if not os.path.exists(folder):
            continue
        # list all files from data updates directory
        if os.path.exists(folder):
            files += [f for f in os.listdir(folder) if os.path.isfile(os.path.join(folder, f))]
    # keep only data updates (00000x*.py)
    files = [f for f in files if DATA_UPDATES_FILENAME_REGEX.match(f)]
    # order file names
    files = sorted(files)
    # remove .py extention if needed
    if strip_file_extension:
        files = [f.rstrip('.py') for f in files]
    return files


class DataUpdateCommand(superdesk.Command):
    """Parent class for Upgrade and Downgrade commands.

    It defines options and initialize some variables in `run` method.
    """

    option_list = [
        superdesk.Option('--id', '-i', dest='data_update_id', required=False,
                         choices=get_data_updates_files(strip_file_extension=True),
                         help='Data update id to run last'),
        superdesk.Option('--fake-init', dest='fake', required=False, action='store_true',
                         help='Mark data updates as run without actually running them'),
        superdesk.Option('--dry-run', dest='dry', required=False, action='store_true',
                         help='Does not mark data updates as done. This can be useful for development.'),
    ]

    def get_applied_updates(self):
        req = ParsedRequest()
        req.sort = '-name'
        return tuple(self.data_updates_service.get(req=req, lookup={}))

    def run(self, data_update_id=None, fake=False, dry=False):
        self.data_updates_service = superdesk.get_resource_service('data_updates')
        self.data_updates_files = get_data_updates_files(strip_file_extension=True)
        # retrieve existing data updates in database
        data_updates_applied = self.get_applied_updates()
        self.last_data_update = data_updates_applied and data_updates_applied[-1] or None
        if self.last_data_update:
            if self.last_data_update['name'] not in self.data_updates_files:
                print('A data update previously applied to this database (%s) can\'t be found in %s' % (
                      self.last_data_update['name'], ', '.join(get_dirs())))

    def compile_update_in_module(self, data_update_name):
        date_update_script_file = None
        for folder in get_dirs():
            date_update_script_file = os.path.join(folder, '%s.py' % (data_update_name))
            if os.path.exists(date_update_script_file):
                break
        assert date_update_script_file is not None, 'File %s has not been found' % (data_update_name)
        # create a module instance to use as scope for our data update
        module = ModuleType('data_update_module')
        with open(date_update_script_file) as f:
            # compile data update script file
            script = compile(f.read(), date_update_script_file, 'exec')
            # excecute the script in the module
            exec(script, module.__dict__)
        return module

    def in_db(self, update):
        return update in map(lambda _: _['name'], self.get_applied_updates())


class Upgrade(DataUpdateCommand):
    """Runs all the new data updates available.

    If `data_update_id` is given, runs new data updates until the given one.
    """

    def run(self, data_update_id=None, fake=False, dry=False):
        super().run(data_update_id, fake, dry)
        data_updates_files = self.data_updates_files
        # drops updates that already have been applied
        data_updates_files = [update for update in data_updates_files if not self.in_db(update)]
        # drop versions after given one
        if data_update_id:
            if data_update_id not in data_updates_files:
                print('Given data update id not found in available updates. It may have been already applied')
                return False
            data_updates_files = data_updates_files[:data_updates_files.index(data_update_id) + 1]
        # apply data updates
        for data_update_name in data_updates_files:
            print('data update %s running forward...' % (data_update_name))
            module_scope = self.compile_update_in_module(data_update_name)
            # run the data update forward
            if not fake:
                module_scope.DataUpdate().apply('forwards')
            if not dry:
                # store the applied data update in the database
                self.data_updates_service.create([{'name': data_update_name}])
        if not data_updates_files:
            print('No data update to apply.')


class Downgrade(DataUpdateCommand):
    """Runs the latest data update backward.

    If `data_update_id` is given, runs all the data updates backward until the given one.
    """

    def run(self, data_update_id=None, fake=False, dry=False):
        super().run(data_update_id, fake, dry)
        data_updates_files = self.data_updates_files
        # check if there is something to downgrade
        if not self.last_data_update:
            print('No data update has been already applied')
            return False
        # drops updates which have not been already made (this is rollback mode)
        data_updates_files = [update for update in data_updates_files if self.in_db(update)]

        # if data_update_id is given, go until this update (drop previous updates)
        if data_update_id:
            if data_update_id not in data_updates_files:
                print('Update %s can\'t be find. It may have been already downgraded' % (data_update_id))
                return False
            data_updates_files = data_updates_files[data_updates_files.index(data_update_id):]
        # otherwise, just rollback one update
        else:
            print('No data update id has been provided. Dowgrading to previous version: %s'
                  % self.last_data_update['name'])
            data_updates_files = data_updates_files[len(data_updates_files) - 1:]
        # apply data updates, in the opposite direction
        for data_update_name in reversed(data_updates_files):
            print('data update %s running backward...' % (data_update_name))
            module_scope = self.compile_update_in_module(data_update_name)
            # run the data update backward
            if not fake:
                module_scope.DataUpdate().apply('backwards')
            if not dry:
                # remove the applied data update from the database
                self.data_updates_service.delete({'name': data_update_name})
        if not data_updates_files:
            print('No data update to apply.')


class GenerateUpdate(superdesk.Command):
    """Generate a file where to define a new data update."""

    option_list = [
        superdesk.Option('--resource', '-r', dest='resource_name', required=True,
                         help='Resource to update'),
        superdesk.Option('--global', '-g', dest='global_update', required=False, action='store_true',
                         help='This data update belongs to superdesk core'),
    ]

    def run(self, resource_name, global_update=False):
        timestamp = time.strftime('%Y%m%d-%H%M%S')
        # create a data update file
        try:
            last_file = get_data_updates_files()[-1]
            name_id = int(last_file.replace('.py', '').split('_')[0]) + 1
        except IndexError:
            name_id = 0
        if global_update:
            update_dir = MAIN_DATA_UPDATES_DIR
        else:
            update_dir = get_dirs(only_relative_folder=True)[0]
        if not os.path.exists(update_dir):
            os.makedirs(update_dir)
        data_update_filename = os.path.join(update_dir, '{:05d}_{}_{}.py'.format(name_id, timestamp, resource_name))
        if os.path.exists(data_update_filename):
            raise Exception('The file "%s" already exists' % (data_update_filename))
        with open(data_update_filename, 'w+') as f:
            template_context = {
                'resource': resource_name,
                'current_date': time.strftime("%Y-%m-%d %H:%M"),
                'user': getpass.getuser(),
                'default_fw_implementation': DEFAULT_DATA_UPDATE_FW_IMPLEMENTATION,
                'default_bw_implementation': DEFAULT_DATA_UPDATE_BW_IMPLEMENTATION
            }
            f.write(Template(DATA_UPDATE_TEMPLATE).substitute(template_context))
            print('Data update file created %s' % (data_update_filename))


superdesk.command('data:generate_update', GenerateUpdate())
superdesk.command('data:upgrade', Upgrade())
superdesk.command('data:downgrade', Downgrade())


class DataUpdate:

    def apply(self, direction):
        assert(direction in ['forwards', 'backwards'])
        collection = current_app.data.get_mongo_collection(self.resource)
        db = current_app.data.driver.db
        getattr(self, direction)(collection, db)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.resource import Resource
from content_api import MONGO_PREFIX, ELASTIC_PREFIX
from content_api.items.resource import schema


class SearchResource(Resource):

    item_url = 'regex("[\w,.:-]+")'
    schema = schema

    datasource = {
        'search_backend': 'elastic',
        'source': 'items',
        'elastic_filter': {"bool": {"must_not": {"term": {"type": "composite"}}}},
        'default_sort': [('_updated', -1)],
        'aggregations': {
            'type': {'terms': {'field': 'type'}},
            'category': {'terms': {'field': 'service.name', 'size': 100}},
            'source': {'terms': {'field': 'source', 'size': 100}},
            'urgency': {'terms': {'field': 'urgency'}},
            'priority': {'terms': {'field': 'priority'}},
            'genre': {'terms': {'field': 'genre.name', 'size': 100}},
        }
    }

    item_methods = ['GET']
    resource_methods = ['GET']
    mongo_prefix = MONGO_PREFIX
    elastic_prefix = ELASTIC_PREFIX
    privileges = {'GET': 'search_capi'}



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013 - 2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


from superdesk.io.feed_parsers.stt_newsml import STTNewsMLFeedParser
from superdesk.tests import TestCase
from lxml import etree
import os


class BaseSTTNewsMLTestCase(TestCase):
    def setUp(self):
        dirname = os.path.dirname(os.path.realpath(__file__))
        fixture = os.path.normpath(os.path.join(dirname, '../fixtures', self.filename))
        provider = {'name': 'Test'}
        with open(fixture, 'rb') as f:
            parser = STTNewsMLFeedParser()
            self.xml_root = etree.parse(f).getroot()
            self.item = parser.parse(self.xml_root, provider)


class STTTestCase(BaseSTTNewsMLTestCase):
    filename = 'stt_newsml_test.xml'

    def test_can_parse(self):
        self.assertTrue(STTNewsMLFeedParser().can_parse(self.xml_root))

    def test_content(self):
        item = self.item[0]
        self.assertEqual(item['headline'], 'Parliament passed the Alcohol Act and the government gained confidence'
                                           '*** TRANSLATED ***')
        self.assertEqual(item['guid'], 'urn:newsml:stt.fi:20171219:101801633:4')
        self.assertEqual(item['uri'], 'urn:newsml:stt.fi:20171219:101801633')
        self.assertEqual(item['firstcreated'].isoformat(), '2017-12-19T10:33:04+02:00')
        expected_body = ('<p>*** DISCLAIMER: THIS IS AN AUTOMATED TRANSLATION FROM FINNISH ***</p>\n\n<h2>Also the une'
                         'mployment law went through</h2>\n\n<p>The Parliament has approved the Alcohol Act 124-65. Th'
                         'ere were two absentee voices and eight MPs absent. As early as Friday, Parliament voted for '
                         'the percentage of alcohol in the grocery store to rise to 5.5.</p>\n\n<p>Government parties '
                         'have previously agreed that the tax on alcohol will be tightened by EUR 100 million, as the '
                         'maximum percentage of alcoholic beverages sold in the store will increase.</p>\n\n<p>In addi'
                         'tion to the alcohol percentage, the Alcohol Act includes a significant number of reforms. Wi'
                         'th the new Alcohol Act, the restaurant can, for example, continue to dispense with a notice '
                         'until 4 o&amp;#39;clock. The restaurant may be open after one hour of serving.</p>\n\n<p>In '
                         'the future, food stores may also sell non-fermented alcoholic beverages, so-called limousine'
                         's. The creatures can be open on weekdays for an hour longer, or at night.</p>\n\n<p>Parliame'
                         'nt also approved the proposal by Hannakaisa Heikkinen (central) that the government should c'
                         'larify the rules on distance selling of alcohol. The distance selling was left unclear in th'
                         'e bill and in the report on it. The family and basic minister Annika Saarikko (center) has a'
                         'lready announced that she will set up a distance marketing team to investigate.</p>\n\n<p>&a'
                         'mp;quot;The will of the government is that we do not want to break Alko&amp;#39;s monopoly w'
                         'ith any of the risk factors that distance selling could include. If the sales of spirits in '
                         'Finland are limitedAlkoon, the same applies to foreign journalists, Saarikko said on Friday.'
                         '</p>\n\n<p>Distance selling means the purchase of alcohol from another EU Member State so th'
                         'at the seller is responsible for the transport of drinks or the organization of transport to'
                         ' the purchaser.</p>\n\n<p>Parliament also voted on the government&amp;#39;s confidence. The '
                         'government gained confidence by 106-80. Seven MPs were absent and six were absent.</p>\n\n<p'
                         '></p>\n<p>\n</p>\n<h3>Rinne: Hallitus runnoi läpi työttömiä kurittavan lain</h3>\n\n\n<p></p'
                         '>\n\n<p>In the morning&amp;#39;s vote, for example, a controversial unemployed security law '
                         'passed through 103-90. There were two absentee voices and four MPs absent.</p>\n\n<p>The opp'
                         'osition has strongly opposed, in particular, the law-related active model, according to whic'
                         'h 4.65 percent of the unemployment allowance is reduced if the unemployed person does not qu'
                         'alify for an adequate amount of paid employment or participates in employment promotion serv'
                         'ices over a period of three months.</p>\n\n<p>- The government parties are running the unemp'
                         'loyed through a law to cure, despite warnings. We do not accept that the unemployed are puni'
                         'shed for the fact that there is simply no job. That is why the SDP will change the active mo'
                         'del, Rinne said in a briefing after the vote.</p>\n\n<p></p>\n<p>*** DISCLAIMER: DETTA ÄR EN'
                         ' AUTOMATISK ÖVERSÄTTNING FRÅN FINNISH ***</p>\n<p>Parlamentet har godkänt alkohollagen 124-6'
                         '5. Det fanns två frånvarande röster och åtta ledamöter frånvarande. Redan i fredags röstade '
                         'parlamentet för att andelen alkohol i livsmedelsbutiken skulle stiga till 5,5.</p>\n<p>Reger'
                         'ingspartierna har tidigare kommit överens om att alkoholskatten kommer att stramas med 100 m'
                         'iljoner euro, eftersom den högsta andelen alkoholhaltiga drycker som säljs i affären kommer '
                         'att öka.</p>\n<p>Förutom alkoholprocenten innehåller alkohollagen ett betydande antal reform'
                         'er. Med den nya alkohollagen kan restaurangen till exempel fortsätta att meddela ett meddela'
                         'nde till klockan 4. Restaurangen kan vara öppen efter en timmes servering.</p>\n<p>I framtid'
                         'en kan mataffärer också sälja alkoholfria drycker, så kallade limousiner. Varelserna kan var'
                         'a öppna på vardagar för en timme längre eller på natten.</p>\n<p>Parlamentet godkände också '
                         'förslaget från Hannakaisa Heikkinen (central) att regeringen skulle klargöra reglerna för di'
                         'stansförsäljning av alkohol. Distansförsäljningen lämnades oklart i propositionen och rappor'
                         'ten om den. Familjen och grundminister Annika Saarikko (center) har redan meddelat att hon k'
                         'ommer att inrätta ett distansmarknadsföringslag att undersöka.</p>\n<p>&amp;quot;Regeringens'
                         ' vilja är att vi inte vill bryta Alkos monopol med någon av de riskfaktorer som distansförsä'
                         'ljningen kan innefatta. Om försäljningen av sprit i Finland är begränsad</p>\n<p>, detsamma '
                         'gäller utländska journalister, sa Saarikko på fredag.</p>\n<p>Distansförsäljning innebär ink'
                         'öp av alkohol från en annan EU-medlemsstat så att säljaren är ansvarig för transport av dryc'
                         'ker eller transporten till köparen.</p>\n<p>Parlamentet röstade också om regeringens förtroe'
                         'nde. Regeringen fick förtroende med 106-80. Sju parlamentsledamöter var frånvarande och sex '
                         'var frånvarande.</p>\n<p></p>\n<p></p>\n<p>På morgonens omröstning gick exempelvis en kontro'
                         'versiell arbetslös säkerhetslag genom 103-90. Det fanns två frånvarande röster och fyra leda'
                         'möter frånvarande.</p>\n<p>Oppositionen har starkt motsatt sig, i synnerhet den lagrelaterad'
                         'e aktiva modellen, enligt vilken 4,65 procent av arbetslöshetsersättningen minskas om den ar'
                         'betslösa inte kvalificerar sig för tillräcklig betald anställning eller deltar i arbetsförme'
                         'dlingstjänster under en period på tre månader.</p>\n<p>- Regeringspartierna driver de arbets'
                         'lösa genom en lag att bota, trots varningar. Vi accepterar inte att de arbetslösa straffas f'
                         'ör att det bara inte finns något jobb. Det är därför som SDP kommer att ändra den aktiva mod'
                         'ellen, sade Rinne i en briefing efter omröstningen.</p>\n<p>*** DISCLAIMER: THIS IS AN AUTOM'
                         'ATED TRANSLATION FROM FINNISH ***</p>\n<p>Eduskunta on hyväksynyt alkoholilain äänin 124–65.'
                         ' Tyhjää äänesti kaksi ja poissa oli kahdeksan kansanedustajaa. Jo perjantaina eduskunta ääne'
                         'sti, että ruokakaupassa myytävän alkoholin prosenttiraja nousee 5,5:een. </p>\n<p> Hallitusp'
                         'uolueet ovat aiemmin sopineet, että alkoholiveroa kiristetään 100 miljoonalla eurolla, kun k'
                         'aupassa myytävien alkoholijuomien enimmäisprosentti nousee.</p>\n<p>Alkoholilakiin sisältyy '
                         'alkoholiprosentin lisäksi merkittävä määrä uudistuksia. Uuden alkoholilain myötä ravintola v'
                         'oi esimerkiksi jatkaa anniskelua pelkällä ilmoituksella kello neljään saakka. Ravintola voi '
                         'olla auki tunnin anniskelun päättymisen jälkeen.</p>\n<p>Jatkossa ruokakaupassa saa myydä my'
                         'ös muutoin kuin käymisteitse valmistettuja alkoholijuomia eli niin sanottuja limuviinoja. Al'
                         'kot saavat olla arkisin auki tunnin pidempään eli iltayhdeksään.</p>\n<p>Eduskunta hyväksyi '
                         'lisäksi Hannakaisa Heikkisen (kesk.) lausumaehdotuksen, jonka mukaan hallituksen on selvitet'
                         'tävä alkoholin etämyyntiä koskevat säännökset. Etämyynti jäi lakiesityksessä ja sitä koskeva'
                         'ssa mietinnössä epäselväksi. Perhe- ja peruspalveluministeri Annika Saarikko (kesk.) on jo k'
                         'ertonut asettavansa työryhmän etämyyntiä selvittämään.</p>\n<p>– Hallituksen tahdonilmaus on'
                         ', että emme halua murtaa Alkon monopolia millään riskitekijöillä, joita etämyynti voisi pitä'
                         'ä sisällään. Jos Suomessa väkevien alkoholijuomien myynti on rajoitettu </p>\n<p>, sama kosk'
                         'ee myös ulkomaisia toimittajia, Saarikko sanoi perjantaina.</p>\n<p>Etämyynti tarkoittaa alk'
                         'oholin hankintaa toisesta EU:n jäsenvaltiosta siten, että myyjä vastaa juomien kuljetuksesta'
                         ' tai kuljetuksen järjestämisestä ostajalle.</p>\n<p>Eduskunta äänesti myös hallituksen luott'
                         'amuksesta. Hallitus sai luottamuksen äänin 106–80. Tyhjää äänesti seitsemän kansanedustajaa '
                         'ja poissa oli kuusi.</p>\n<p>\n</p>\n<p>\n</p>\n<p>Aamupäivän äänestyksissä meni läpi myös e'
                         'simerkiksi kiistelty työttömyysturvalaki äänin 103–90. Tyhjää äänesti kaksi ja poissa oli ne'
                         'ljä kansanedustajaa. </p>\n<p>Oppositio on vastustanut jyrkästi etenkin lakiin liittyvää akt'
                         'iivimallia, jonka mukaan työttömyyskorvauksesta nipistetään 4,65 prosenttia, jos työtön ei k'
                         'äy riittävää määrää palkkatyössä tai osallistu työllistymistä edistäviin palveluihin kolmen '
                         'kuukauden aikana.</p>\n<p>–\u2009 Hallituspuolueet runnoivat työttömiä kurittavan lain läpi '
                         'varoituksista huolimatta. Me emme hyväksy sitä, että työttömiä rangaistaan siitä, että työtä'
                         ' ei yksinkertaisesti ole. Siksi SDP tulee muuttamaan aktiivimallin, Rinne sanoi tiedotteessa'
                         'an heti äänestyksen jälkeen.</p>')

        self.assertEqual(item['body_html'], expected_body)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import re
import hashlib
from dateutil import parser

from superdesk.errors import IngestTwitterError
from superdesk.io.registry import register_feeding_service
from superdesk.io.feeding_services import FeedingService
from superdesk.metadata.utils import generate_guid
from superdesk.metadata.item import ITEM_TYPE, CONTENT_TYPE, GUID_TAG

import twitter


class TwitterFeedingService(FeedingService):
    """
    Feeding Service class which can read feed from multiple Twitter
    Screen Names and also hashtags
    """

    NAME = 'twitter'
    label = 'Twitter'

    ERRORS = [IngestTwitterError.TwitterLoginError().get_error_description(),
              IngestTwitterError.TwitterNoScreenNamesError().
              get_error_description()]

    def _test(self, provider):
        self._update(provider, update=None, test=True)

    def _update(self, provider, update, test=False):
        config = provider.get('config', {})
        consumer_key = config.get('consumer_key', '')
        consumer_secret = config.get('consumer_secret', '')
        access_token_key = config.get('access_token_key', '')
        access_token_secret = config.get('access_token_secret', '')
        screen_names = config.get('screen_names', '')
        status_count = 100
        # how many statuses to get, 200 should be max

        new_items = []
        api = twitter.Api(consumer_key=consumer_key,
                          consumer_secret=consumer_secret,
                          access_token_key=access_token_key,
                          access_token_secret=access_token_secret)
        try:
            screen_names = screen_names.split(',')
        except Exception as ex:
            raise IngestTwitterError.TwitterNoScreenNamesError(ex, provider)

        if not screen_names:
            raise IngestTwitterError.TwitterNoScreenNamesError(provider)
        for screen_name in screen_names:
            screen_name = screen_name.replace(' ', '')
            try:
                # hashtag search
                if screen_name.startswith('#'):
                    statuses = api.GetSearch(screen_name.lstrip('#'),
                                             count=status_count)
                # user search
                else:
                    statuses = api.GetUserTimeline(screen_name=screen_name,
                                                   count=status_count)
            except twitter.error.TwitterError as exc:
                if exc.message[0].get('code') == 34:
                    # that page does not exist
                    continue
                elif exc.message[0].get('code') == 32:
                    # invalid credentials
                    raise IngestTwitterError.TwitterLoginError(exc, provider)

            for status in statuses:
                d = parser.parse(status.created_at)
                guid_hash = hashlib.sha1(status.text.
                                         encode('utf8')).hexdigest()
                guid = generate_guid(type=GUID_TAG, id=guid_hash)
                headline = "%s: %s" % (status.user.screen_name, status.text)
                item = {}
                item['headline'] = headline
                item['type'] = 'text'
                item['guid'] = guid
                item['versioncreated'] = d
                item['firstcreated'] = d

                item['body_html'] = status.text
                # include URL on body
                urls = re.findall('http[s]?://(?:[a-zA-Z]|[0-9]|[$-_@.&+]|'
                                  '[!*\(\),]|(?:%[0-9a-fA-F][0-9a-fA-F]))+',
                                  status.text)
                if urls:
                    item['body_html'] += '<p><a href="%s"' \
                        ' target="_blank">%s</a></p>' % (urls[0], urls[0])

                # on hashtag search we don't want retweets
                if not (screen_name.startswith('#') and status.text
                        .startswith('RT ')):
                    new_items.append(item)
                    if status.media:
                        image_urls = []
                        for media_item in status.media:
                            if media_item.type in ['photo', 'animated',
                                                   'video']:
                                image_urls.append(media_item.media_url)
                        try:
                            # Eg can fail while fetching image
                            image_items = self._create_image_items(image_urls,
                                                                   item)
                            new_items.extend(image_items)
                            package_item = self._create_package(item,
                                                                image_items)
                            new_items.append(package_item)
                        except Exception as ex:
                            pass
        return [new_items]

    def _create_package(self, text_item, image_items):
        """
        Create a new content package from given content items.
        """
        package = {
            ITEM_TYPE: CONTENT_TYPE.COMPOSITE,
            'guid': generate_guid(type=GUID_TAG,
                                  id=text_item.get('guid') + '-package'),
            'versioncreated': text_item['versioncreated'],
            'firstcreated': text_item.get('firstcreated'),
            'headline': text_item.get('headline', ''),
            'groups': [
                {
                    'id': 'root',
                    'role': 'grpRole:NEP',
                    'refs': [{'idRef': 'main'}],
                }, {
                    'id': 'main',
                    'role': 'main',
                    'refs': [],
                }
            ]
        }

        item_references = package['groups'][1]['refs']
        item_references.append({'residRef': text_item['guid']})

        for image in image_items:
            item_references.append({'residRef': image['guid']})

        return package

    def _create_image_items(self, image_links, text_item):
        image_items = []

        for image_url in image_links:
            guid_hash = hashlib.sha1(image_url.encode('utf8')).hexdigest()
            img_item = {
                'guid': generate_guid(type=GUID_TAG,
                                      id=text_item.get('guid') +
                                      guid_hash + '-image'),
                ITEM_TYPE: CONTENT_TYPE.PICTURE,
                'versioncreated': text_item.get('versioncreated'),
                'firstcreated': text_item.get('firstcreated'),
                'headline': text_item.get('headline', ''),
                'renditions': {
                    'baseImage': {
                        'href': image_url
                    }
                }
            }
            image_items.append(img_item)

        return image_items


register_feeding_service(TwitterFeedingService.NAME, TwitterFeedingService(),
                         TwitterFeedingService.ERRORS)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013 - 2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import arrow
import datetime
import logging

from superdesk.errors import ParserError
from superdesk.io.registry import register_feed_parser
from superdesk.io.feed_parsers import XMLFeedParser
from superdesk.io.iptc import subject_codes
from superdesk.metadata.item import ITEM_TYPE, CONTENT_TYPE
from superdesk.metadata.utils import is_normal_package

XMLNS = 'http://iptc.org/std/nar/2006-10-01/'
XHTML = 'http://www.w3.org/1999/xhtml'

logger = logging.getLogger(__name__)


class NewsMLTwoFeedParser(XMLFeedParser):
    """
    Feed Parser which can parse if the feed is in NewsML 2 format.
    """

    NAME = 'newsml2'

    label = 'News ML 2.0 Parser'
    SUBJ_QCODE_PREFIXES = ('subj',)

    def can_parse(self, xml):
        return xml.tag.endswith('newsMessage')

    def parse(self, xml, provider=None):
        self.root = xml
        items = []
        try:
            header = self.parse_header(xml)
            for item_set in xml.findall(self.qname('itemSet')):
                for item_tree in item_set:
                    item = self.parse_item(item_tree)
                    item['priority'] = header['priority']
                    items.append(item)
            return items
        except Exception as ex:
            raise ParserError.newsmlTwoParserError(ex, provider)

    def parse_item(self, tree):
        item = dict()
        item['guid'] = tree.attrib['guid'] + ':' + tree.attrib['version']
        item['uri'] = tree.attrib['guid']
        item['version'] = tree.attrib['version']

        self.parse_item_meta(tree, item)
        self.parse_content_meta(tree, item)
        self.parse_rights_info(tree, item)

        if is_normal_package(item):
            self.parse_group_set(tree, item)
        else:
            self.parse_content_set(tree, item)

        return item

    def parse_header(self, tree):
        """Parse header element.

        :param tree:
        :return: dict
        """
        header = tree.find(self.qname('header'))
        priority = 5
        if header is not None:
            priority = self.map_priority(header.find(self.qname('priority')).text)

        return {'priority': priority}

    def parse_item_meta(self, tree, item):
        """Parse itemMeta tag"""
        meta = tree.find(self.qname('itemMeta'))
        item[ITEM_TYPE] = meta.find(self.qname('itemClass')).attrib['qcode'].split(':')[1]
        item['versioncreated'] = self.datetime(meta.find(self.qname('versionCreated')).text)
        firstcreated_elt = meta.find(self.qname('firstCreated'))
        if firstcreated_elt is not None:
            item['firstcreated'] = self.datetime(firstcreated_elt.text)
        item['pubstatus'] = (meta.find(self.qname('pubStatus')).attrib['qcode'].split(':')[1]).lower()
        item['ednote'] = meta.find(self.qname('edNote')).text if meta.find(self.qname('edNote')) is not None else ''

    def parse_content_meta(self, tree, item):
        """Parse contentMeta tag"""
        meta = tree.find(self.qname('contentMeta'))

        def parse_meta_item_text(key, dest=None, elemTree=None):
            if dest is None:
                dest = key

            if elemTree is None:
                elemTree = meta

            elem = elemTree.find(self.qname(key))
            if elem is not None:
                if dest == 'urgency':
                    item[dest] = int(elem.text)
                else:
                    item[dest] = elem.text

        parse_meta_item_text('urgency')
        parse_meta_item_text('slugline')
        parse_meta_item_text('headline')
        # parse_meta_item_text('creditline')
        parse_meta_item_text('by', 'byline')

        item['slugline'] = item.get('slugline', '')
        item['headline'] = item.get('headline', '')

        try:
            item['description_text'] = meta.find(self.qname('description')).text
            item['archive_description'] = item['description_text']
        except AttributeError:
            pass

        try:
            item['language'] = meta.find(self.qname('language')).get('tag')
        except AttributeError:
            pass

        self.parse_content_subject(meta, item)
        self.parse_content_place(meta, item)

        for info_source in meta.findall(self.qname('infoSource')):
            if info_source.get('role', '') == 'cRole:source':
                item['original_source'] = info_source.get('literal')
                break

        item['genre'] = []
        for genre_el in meta.findall(self.qname('genre')):
            for name_el in genre_el.findall(self.qname('name')):
                lang = name_el.get(self.qname("lang", ns='xml'))
                if lang and lang.startswith('en'):
                    item['genre'].append({'name': name_el.text})

    def parse_content_subject(self, tree, item):
        """Parse subj type subjects into subject list."""
        item['subject'] = []
        for subject in tree.findall(self.qname('subject')):
            qcode_parts = subject.get('qcode', '').split(':')
            if len(qcode_parts) == 2 and qcode_parts[0] in self.SUBJ_QCODE_PREFIXES:
                try:
                    item['subject'].append({
                        'qcode': qcode_parts[1],
                        'name': subject_codes[qcode_parts[1]]
                    })
                except KeyError:
                    logger.debug("Subject code '%s' not found" % qcode_parts[1])

    def parse_content_place(self, tree, item):
        """Parse subject with type="cptType:5" into place list."""
        for subject in tree.findall(self.qname('subject')):
            if subject.get('type', '') == 'cptType:5':
                item['place'] = []
                item['place'].append({'name': self.get_literal_name(subject)})
                broader = subject.find(self.qname('broader'))
                if broader is not None:
                    item['place'].append({'name': self.get_literal_name(broader)})

    def parse_rights_info(self, tree, item):
        """Parse Rights Info tag"""
        info = tree.find(self.qname('rightsInfo'))
        if info is not None:
            item['usageterms'] = getattr(info.find(self.qname('usageTerms')), 'text', '')
            # item['copyrightholder'] = info.find(self.qname('copyrightHolder')).attrib['literal']
            # item['copyrightnotice'] = getattr(info.find(self.qname('copyrightNotice')), 'text', None)

    def parse_group_set(self, tree, item):
        item['groups'] = []
        for group in tree.find(self.qname('groupSet')):
            data = {}
            data['id'] = group.attrib['id']
            data['role'] = group.attrib['role']
            data['refs'] = self.parse_refs(group)
            item['groups'].append(data)

    def parse_refs(self, group_tree):
        refs = []
        for tree in group_tree:
            if 'idref' in tree.attrib:
                refs.append({'idRef': tree.attrib['idref']})
            else:
                ref = {}
                if 'version' in tree.attrib:
                    ref['residRef'] = tree.attrib['residref'] + ':' + tree.attrib['version']
                else:
                    ref['residRef'] = tree.attrib['residref']
                ref['contentType'] = tree.attrib['contenttype']
                ref['itemClass'] = tree.find(self.qname('itemClass')).attrib['qcode']

                for headline in tree.findall(self.qname('headline')):
                    ref['headline'] = headline.text

                refs.append(ref)
        return refs

    def parse_content_set(self, tree, item):
        item['renditions'] = {}
        for content in tree.find(self.qname('contentSet')):
            if content.tag == self.qname('inlineXML'):
                try:
                    item['word_count'] = int(content.attrib['wordcount'])
                except KeyError:
                    pass
                content = self.parse_inline_content(content, item=item)
                item['body_html'] = content.get('content')
                if 'format' in content:
                    item['format'] = content.get('format')
            elif content.tag == self.qname('inlineData'):
                item['body_html'] = content.text
                try:
                    item['word_count'] = int(content.attrib['wordcount'])
                except KeyError:
                    pass
            else:
                rendition = self.parse_remote_content(content)
                item['renditions'][rendition['rendition']] = rendition

    def parse_inline_content(self, tree, item, ns=XHTML):
        html = tree.find(self.qname('html', ns))
        body = html.find(self.qname('body', ns))
        elements = []
        for elem in body:
            if elem.text:
                tag = elem.tag.rsplit('}')[1]
                elements.append('<%s>%s</%s>' % (tag, elem.text, tag))

        # If there is a single p tag then replace the line feeds with breaks
        if len(elements) == 1 and body[0].tag.rsplit('}')[1] == 'p':
            elements[0] = elements[0].replace('\n    ', '</p><p>').replace('\n', '<br/>')

        content = dict()
        content['contenttype'] = tree.attrib['contenttype']
        if len(elements) > 0:
            content['content'] = "\n".join(elements)
        elif body.text:
            content['content'] = '<pre>' + body.text + '</pre>'
            content['format'] = CONTENT_TYPE.PREFORMATTED
        return content

    def parse_remote_content(self, tree):
        content = dict()
        content['residRef'] = tree.attrib.get('residref')
        content['sizeinbytes'] = int(tree.attrib.get('size', '0'))
        content['rendition'] = tree.attrib['rendition'].split(':')[1]
        content['mimetype'] = tree.attrib['contenttype']
        content['href'] = tree.attrib.get('href', None)
        return content

    def datetime(self, string):
        try:
            return datetime.datetime.strptime(string, '%Y-%m-%dT%H:%M:%S.000Z')
        except ValueError:
            return arrow.get(string).datetime

    def get_literal_name(self, item):
        """Get name for item with fallback to literal attribute if name is not provided."""
        name = item.find(self.qname('name'))
        return name.text if name is not None else item.attrib.get('literal')


register_feed_parser(NewsMLTwoFeedParser.NAME, NewsMLTwoFeedParser())



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.resource import Resource
from content_api import MONGO_PREFIX

schema = {
    'type': {'type': 'string'},
    'subscriber': {'type': 'string'},
    'uri': {'type': 'string'},
    'items_id': {'type': 'string'},
    'version': {'type': 'string'},
    'remote_addr': {'type': 'string'}
}


class ApiAuditResource(Resource):
    """
    Resource class for the auditing of the API
    """

    schema = schema
    mongo_prefix = MONGO_PREFIX
    internal_resource = True
    mongo_indexes = {'subscriber': [('subscriber', 1)]}



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.resource import Resource
from content_api import MONGO_PREFIX
from content_api.items.resource import schema
from copy import deepcopy


class PackagesVersionsResource(Resource):
    item_url = 'regex("[\w,.:-]+")'
    version_schema = deepcopy(schema)
    version_schema['_id_document'] = {'type': 'string'}
    schema = version_schema

    datasource = {'source': 'items_versions'}

    item_methods = ['GET']
    resource_methods = ['GET']
    internal_resource = True
    mongo_prefix = MONGO_PREFIX



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import superdesk


class ValidatorsResource(superdesk.Resource):
    schema = {
        '_id': {'type': 'string', 'required': True, 'unique': True},
        'act': {'type': 'string', 'required': True},
        'type': {'type': 'string', 'required': True},
        'embedded': {'type': 'boolean', 'required': False},
        'schema': {
            'type': 'dict',
            'required': False
        }
    }

    resource_methods = ['POST']
    item_methods = []


class ValidatorsService(superdesk.Service):
    pass



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from eve.versioning import resolve_document_version
from .common import BasePublishService, BasePublishResource, ITEM_KILL
from eve.utils import config
from superdesk.metadata.item import CONTENT_STATE, ITEM_STATE, PUB_STATUS, EMBARGO, SCHEDULE_SETTINGS, \
    PUBLISH_SCHEDULE
from superdesk import get_resource_service
from superdesk.utc import utcnow
import logging
from copy import deepcopy
from superdesk.emails import send_article_killed_email
from superdesk.errors import SuperdeskApiError
from apps.archive.common import ITEM_OPERATION, ARCHIVE, insert_into_versions, get_dateline_city
from itertools import chain

logger = logging.getLogger(__name__)


class KillPublishResource(BasePublishResource):
    def __init__(self, endpoint_name, app, service):
        super().__init__(endpoint_name, app, service, 'kill')


class KillPublishService(BasePublishService):
    publish_type = 'kill'
    published_state = 'killed'

    def __init__(self, datasource=None, backend=None):
        super().__init__(datasource=datasource, backend=backend)

    def on_update(self, updates, original):
        # check if we are trying to kill and item that is contained in package
        # and the package itself is not killed.

        packages = self.package_service.get_packages(original[config.ID_FIELD])
        if packages and packages.count() > 0:
            for package in packages:
                if package[ITEM_STATE] != CONTENT_STATE.KILLED:
                    raise SuperdeskApiError.badRequestError(message='This item is in a package. '
                                                                    'It needs to be removed '
                                                                    'before the item can be killed')

        updates['pubstatus'] = PUB_STATUS.CANCELED
        updates['versioncreated'] = utcnow()

        super().on_update(updates, original)
        updates[ITEM_OPERATION] = ITEM_KILL
        get_resource_service('archive_broadcast').spike_item(original)

    def update(self, id, updates, original):
        """Kill will broadcast kill email notice to all subscriber in the system and then kill the item.

        Kill for multiple items is triggered
        - for broadcast items if master item is killed.
        In case of the multiple items the kill header text will be different but rest
        of the body_html will be same.

        If any other fields needs in the kill template that needs to be based on the item that is being
        killed (not the item that is being actioned on) then modify the article_killed_override.json file'
        """
        # kill cannot be scheduled and embargoed.
        updates[EMBARGO] = None
        updates[PUBLISH_SCHEDULE] = None
        updates[SCHEDULE_SETTINGS] = {}
        updates_copy = deepcopy(updates)
        original_copy = deepcopy(original)
        self.apply_kill_override(original_copy, updates)
        self.broadcast_kill_email(original, updates)
        super().update(id, updates, original)
        updated = deepcopy(original)
        updated.update(updates)
        get_resource_service('archive_broadcast').kill_broadcast(updates_copy, original_copy)

    def broadcast_kill_email(self, original, updates):
        """Sends the broadcast email to all subscribers (including in-active subscribers)

        :param dict original: Document to kill
        :param dict updates: kill updates
        """
        # Get all subscribers
        subscribers = list(get_resource_service('subscribers').get(req=None, lookup=None))

        recipients = [s.get('email').split(',') for s in subscribers if s.get('email')]
        recipients = list(set(chain(*recipients)))
        # send kill email.
        kill_article = deepcopy(original)
        kill_article['body_html'] = updates.get('body_html')
        kill_article['headline'] = updates.get('headline')
        kill_article['desk_name'] = get_resource_service('desks').get_desk_name(kill_article.get('task',
                                                                                                 {}).get('desk'))
        kill_article['city'] = get_dateline_city(kill_article.get('dateline'))
        send_article_killed_email(kill_article, recipients, utcnow())

    def kill_item(self, updates, original):
        """Kill the item after applying the template.

        :param dict item: Item
        :param str body_html: body_html of the original item that triggered the kill.
        """
        # apply the kill template
        original_copy = deepcopy(original)
        updates_data = self._apply_kill_template(original_copy)
        updates_data['body_html'] = updates.get('body_html', '')
        # resolve the document version
        resolve_document_version(document=updates_data, resource=ARCHIVE, method='PATCH', latest_doc=original)
        # kill the item
        self.patch(original.get(config.ID_FIELD), updates_data)
        # insert into versions
        insert_into_versions(id_=original[config.ID_FIELD])
